@@ -70,7 +70,10 @@ template<typename T> void add_smat33(py::module& m, const char* name) {
            v.push_back((T)self.r_u_r(Vec3(r(row, 0), r(row, 1), r(row, 2))));
         return py_array_from_vector(std::move(v));
     }, py::arg().noconvert())
-    .def("transformed_by", &M::template transformed_by<double>)
+    .def("multiply", &M::multiply)
+    .def(py::self + py::self)
+    .def(py::self - py::self)
+    .def("transformed_by", &M::template transformed_by<T>)
     .def("calculate_eigenvalues", &M::calculate_eigenvalues)
     .def("__repr__", [name](const M& m) {
         return tostr("<gemmi.", name, '(', m.u11, ", ", m.u22, ", ", m.u33, ", ",
@@ -144,6 +147,7 @@ void add_unitcell(py::module& m) {
     .def("multiply", (Mat33 (Mat33::*)(const Mat33&) const) &Mat33::multiply)
     .def("multiply", (Vec3 (Mat33::*)(const Vec3&) const) &Mat33::multiply)
     .def("left_multiply", &Mat33::left_multiply)
+    .def("multiply_by_diagonal", &Mat33::multiply_by_diagonal)
     .def("transpose", &Mat33::transpose)
     .def("trace", &Mat33::trace)
     .def("approx", &Mat33::approx, py::arg("other"), py::arg("epsilon"))
@@ -270,13 +274,20 @@ void add_unitcell(py::module& m) {
     .def("is_compatible_with_spacegroup", &UnitCell::is_compatible_with_spacegroup,
          py::arg("sg"), py::arg("eps")=1e-3)
     .def("is_crystal", &UnitCell::is_crystal)
+    .def("approx", &UnitCell::approx)
     .def("calculate_u_eq", &UnitCell::calculate_u_eq)
     .def("fractionalize", &UnitCell::fractionalize)
     .def("orthogonalize", &UnitCell::orthogonalize)
     .def("volume_per_image", &UnitCell::volume_per_image)
     .def("find_nearest_image", &UnitCell::find_nearest_image,
          py::arg("ref"), py::arg("pos"), py::arg("asu")=Asu::Any)
-    .def("find_nearest_pbc_image", &UnitCell::find_nearest_pbc_image,
+    .def("find_nearest_pbc_image",
+         (NearestImage (UnitCell::*)(const Fractional&, Fractional, int) const)
+         &UnitCell::find_nearest_pbc_image,
+         py::arg("fref"), py::arg("fpos"), py::arg("image_idx"))
+    .def("find_nearest_pbc_image",
+         (NearestImage (UnitCell::*)(const Position&, const Position&, int) const)
+         &UnitCell::find_nearest_pbc_image,
          py::arg("ref"), py::arg("pos"), py::arg("image_idx"))
     .def("find_nearest_pbc_position", &UnitCell::find_nearest_pbc_position,
          py::arg("ref"), py::arg("pos"), py::arg("image_idx"), py::arg("inverse")=false)
