@@ -27,10 +27,11 @@
 
 namespace {
 
-enum OptionIndex { Hkl=4, Dmin, For, Rate, Blur, RCut, Test, ToMtz, Compare,
-                   CifFp, Wavelength, Unknown, NoAniso, ScaleTo, FLabel,
-                   PhiLabel, Ksolv, Bsolv, Baniso, RadiiSet, Rprobe, Rshrink,
-                   WriteMap };
+enum OptionIndex {
+  Hkl=4, Dmin, For, NormalizeIt92, Rate, Blur, RCut, Test, ToMtz, Compare,
+  CifFp, Wavelength, Unknown, NoAniso, ScaleTo, FLabel,
+  PhiLabel, Ksolv, Bsolv, Baniso, RadiiSet, Rprobe, Rshrink, WriteMap
+};
 
 struct SfCalcArg: public Arg {
   static option::ArgStatus FormFactors(const option::Option& option, bool msg) {
@@ -78,6 +79,8 @@ const option::Descriptor Usage[] = {
     "  --dmin=NUM  \tCalculate structure factors up to given resolution." },
   { For, 0, "", "for", SfCalcArg::FormFactors,
     "  --for=TYPE  \tTYPE is xray (default), electron or mott-bethe." },
+  { NormalizeIt92, 0, "", "normalize-it92", Arg::None,
+    "  --normalize-it92  \tNormalize X-ray form factors (a tiny change)." },
   { CifFp, 0, "", "ciffp", Arg::None,
     "  --ciffp  \tRead f' from _atom_type_scat_dispersion_real in CIF." },
   { Wavelength, 0, "w", "wavelength", Arg::Float,
@@ -187,6 +190,27 @@ void print_to_stderr(const Comparator& c) {
           c.rmse(), 100 * c.weighted_rmse(), c.max_abs_df, 100 * c.rfactor());
 }
 
+template<typename Real>
+void write_asudata_to_mtz(const gemmi::AsuData<std::complex<Real>>& asu_data,
+                          const RefFile& file) {
+  std::unique_ptr<gemmi::Mtz> output_mtz(new gemmi::Mtz(true));
+  output_mtz->set_cell_for_all(asu_data.unit_cell());
+  output_mtz->spacegroup = asu_data.spacegroup();
+  output_mtz->add_dataset("calculated");
+  output_mtz->add_column(file.f_label, 'F');
+  output_mtz->add_column(file.phi_label, 'P');
+  output_mtz->title = "Fcalc generated by gemmi";
+  output_mtz->nreflections = (int) asu_data.v.size();
+  output_mtz->data.reserve(5 * asu_data.v.size());
+  for (const gemmi::HklValue<std::complex<Real>>& hv : asu_data.v) {
+    for (int i = 0; i != 3; ++i)
+      output_mtz->data.push_back((float) hv.hkl[i]);
+    output_mtz->data.push_back((float) std::abs(hv.value));
+    output_mtz->data.push_back((float) gemmi::phase_in_angles(hv.value));
+  }
+  output_mtz->write_to_file(file.path);
+}
+
 template<typename Table, typename Real>
 void process_with_fft(const gemmi::Structure& st,
                       gemmi::DensityCalculator<Table, Real>& dencalc,
@@ -264,27 +288,13 @@ void process_with_fft(const gemmi::Structure& st,
   }
   scaling.scale_data(asu_data, mask_data);
 
-  std::unique_ptr<gemmi::Mtz> output_mtz;
   if (file.mode == RefFile::Mode::WriteMtz) {
-    output_mtz.reset(new gemmi::Mtz(true));
-    output_mtz->set_cell_for_all(asu_data.unit_cell());
-    output_mtz->spacegroup = asu_data.spacegroup();
-    output_mtz->add_dataset("calculated");
-    output_mtz->add_column(file.f_label, 'F');
-    output_mtz->add_column(file.phi_label, 'P');
-    output_mtz->title = "Fcalc generated by gemmi";
-    output_mtz->nreflections = (int) asu_data.v.size();
-    output_mtz->data.reserve(5 * asu_data.v.size());
-  }
-  for (gemmi::HklValue<std::complex<Real>>& hv : asu_data.v) {
-    if (file.mode == RefFile::Mode::None) {
+    write_asudata_to_mtz(asu_data, file);
+  } else if (file.mode == RefFile::Mode::None) {
+    for (gemmi::HklValue<std::complex<Real>>& hv : asu_data.v)
       print_sf(hv.value, hv.hkl);
-    } else if (file.mode == RefFile::Mode::WriteMtz) {
-      for (int i = 0; i != 3; ++i)
-        output_mtz->data.push_back((float) hv.hkl[i]);
-      output_mtz->data.push_back(std::abs(hv.value));
-      output_mtz->data.push_back((float) gemmi::phase_in_angles(hv.value));
-    } else {
+  } else {
+    for (gemmi::HklValue<std::complex<Real>>& hv : asu_data.v) {
       std::complex<double> exact;
       if (file.path) {
         if (file.mode == RefFile::Mode::Test) {
@@ -315,43 +325,50 @@ void process_with_fft(const gemmi::Structure& st,
              gemmi::phase_in_angles(hv.value), gemmi::phase_in_angles(exact),
              sf.unit_cell.calculate_d(hv.hkl));
     }
-  }
-  if (file.mode == RefFile::Mode::Test || file.mode == RefFile::Mode::Compare) {
     print_to_stderr(comparator);
     fprintf(stderr, "  <dPhi>=%#.4g", comparator.mean_dphi());
     if (!verbose)
       fprintf(stderr, "   %#.5gs", timer.count());
     fprintf(stderr, "\n");
-  } else if (file.mode == RefFile::Mode::WriteMtz) {
-    output_mtz->write_to_file(file.path);
   }
 }
 
 template<typename Table>
 void print_structure_factors_sm(const gemmi::SmallStructure& small,
                                 gemmi::StructureFactorCalculator<Table>& calc,
-                                double d_min, bool verbose, bool mott_bethe) {
+                                bool mott_bethe, double d_min, bool verbose,
+                                const RefFile& file) {
   Timer timer(verbose);
   timer.start();
   int counter = 0;
+  // cf. prepare_asu_data()
   double max_1_d = 1. / d_min;
   int max_h = int(max_1_d / small.cell.ar);
   int max_k = int(max_1_d / small.cell.br);
   int max_l = int(max_1_d / small.cell.cr);
   const gemmi::SpaceGroup* sg = small.find_spacegroup();
-  gemmi::ReciprocalAsu asu(sg ? sg : &gemmi::get_spacegroup_p1());
+  if (!sg)
+    sg = &gemmi::get_spacegroup_p1();
+  gemmi::ReciprocalAsu asu(sg);
+  gemmi::AsuData<std::complex<double>> asu_data;
+  gemmi::GroupOps gops = sg->operations();
   for (int h = -max_h; h <= max_h; ++h)
     for (int k = -max_k; k <= max_k; ++k)
       for (int l = 0; l <= max_l; ++l) {
         gemmi::Miller hkl{{h, k, l}};
-        if (!asu.is_in(hkl))
+        if (!asu.is_in(hkl) || (hkl[0] == 0 && hkl[1] == 0 && hkl[2] == 0))
+          continue;
+        if (gops.is_systematically_absent(hkl))
           continue;
         double hkl_1_d2 = small.cell.calculate_1_d2(hkl);
         if (hkl_1_d2 < max_1_d * max_1_d) {
           auto value = calc.calculate_sf_from_small_structure(small, hkl);
           if (mott_bethe)
             value *= calc.mott_bethe_factor();
-          print_sf(value, hkl);
+          if (file.mode == RefFile::Mode::WriteMtz)
+            asu_data.v.push_back({hkl, value});
+          else
+            print_sf(value, hkl);
           ++counter;
         }
       }
@@ -360,6 +377,11 @@ void print_structure_factors_sm(const gemmi::SmallStructure& small,
     fprintf(stderr, "Calculated %d SFs in %g s.\n", counter, timer.count());
     fflush(stderr);
   }
+  if (file.mode == RefFile::Mode::WriteMtz) {
+    asu_data.unit_cell_ = small.cell;
+    asu_data.spacegroup_ = sg;
+    write_asudata_to_mtz(asu_data, file);
+  }
 }
 
 template<typename Table>
@@ -531,11 +553,12 @@ void process_with_table(bool use_st, gemmi::Structure& st, const gemmi::SmallStr
   }
   if (p.options[FLabel])
     file.f_label = p.options[FLabel].arg;
-  else if (use_st)
+  // we leave f_label empty for use in compare_with_hkl()
+  else if (use_st || file.mode == RefFile::Mode::WriteMtz)
     file.f_label = "FC";
   if (p.options[PhiLabel])
    file.phi_label = p.options[PhiLabel].arg;
-  else if (use_st)
+  else
     file.phi_label = "PHIC";
 
   using Real = float;
@@ -634,7 +657,7 @@ void process_with_table(bool use_st, gemmi::Structure& st, const gemmi::SmallStr
           p.options[Test])
         gemmi::fail("Small molecule SFs are calculated directly. Do not use any\n"
                     "of the FFT-related options: --rate, --blur, --rcut, --test.");
-      print_structure_factors_sm(small, calc, d_min, p.options[Verbose], mott_bethe);
+      print_structure_factors_sm(small, calc, mott_bethe, d_min, p.options[Verbose], file);
     }
 
   // handle option --compare
@@ -668,12 +691,17 @@ void process(const std::string& input, const OptParser& p) {
     small.change_occupancies_to_crystallographic();
   }
 
-  if (!p.options[Dmin] || !use_st) {
+  if (!p.options[Dmin]) {
     for (OptionIndex opt : {ToMtz, WriteMap, ScaleTo, Ksolv, Bsolv,
                             RadiiSet, Rprobe, Rshrink})
       if (p.options[opt])
-        gemmi::fail("Option ", p.options[opt].name, " works only with ",
-                    !p.options[Dmin] ? "--dmin" : "macromolecular structures");
+        gemmi::fail("Option ", p.options[opt].name, " works only with --dmin");
+  }
+  if (!use_st) {
+    for (OptionIndex opt : {WriteMap, ScaleTo, Ksolv, Bsolv, RadiiSet, Rprobe, Rshrink})
+      if (p.options[opt])
+        gemmi::fail("Option ", p.options[opt].name, " is only used in density-FFT "
+                    "route which is only used for macromolecular structures");
   }
 
   if (p.options[NoAniso]) {
@@ -723,6 +751,8 @@ void process(const std::string& input, const OptParser& p) {
   if (p.options[CifFp] && table != 'x')
     gemmi::fail("Electron scattering has no dispersive part (--ciffp)");
   if (table == 'x' || table == 'm') {
+    if (p.options[NormalizeIt92])
+      gemmi::IT92<double>::normalize();
     process_with_table<gemmi::IT92<double>>(use_st, st, small, wavelength,
                                             table == 'm', p);
   } else if (table == 'e') {
